@@ -49,12 +49,9 @@
 #elif defined(PIPE_ARCH_PPC_64) || defined(PIPE_ARCH_S390) || defined(PIPE_ARCH_ARM) || defined(PIPE_ARCH_AARCH64)
 #  define USE_MCJIT 1
 #else
-#  define USE_MCJIT 0
+static bool USE_MCJIT = 0;
 #endif
 
-#if USE_MCJIT
-void LLVMLinkInMCJIT();
-#endif
 
 #ifdef DEBUG
 unsigned gallivm_debug = 0;
@@ -69,6 +66,7 @@ static const struct debug_named_value lp_bld_debug_flags[] = {
    { "no_rho_approx", GALLIVM_DEBUG_NO_RHO_APPROX, NULL },
    { "no_quad_lod", GALLIVM_DEBUG_NO_QUAD_LOD, NULL },
    { "gc",     GALLIVM_DEBUG_GC, NULL },
+   { "dumpbc", GALLIVM_DEBUG_DUMP_BC, NULL },
    DEBUG_NAMED_VALUE_END
 };
 
@@ -118,8 +116,10 @@ create_pass_manager(struct gallivm_state *gallivm)
     * simple, or constant propagation into them, etc.
     */
 
+#if HAVE_LLVM < 0x0309
    // Old versions of LLVM get the DataLayout from the pass manager.
    LLVMAddTargetData(gallivm->target, gallivm->passmgr);
+#endif
 
    /* Setting the module's DataLayout to an empty string will cause the
     * ExecutionEngine to copy to the DataLayout string from its target
@@ -188,13 +188,15 @@ gallivm_free_ir(struct gallivm_state *gallivm)
       LLVMDisposeModule(gallivm->module);
    }
 
-#if !USE_MCJIT
-   /* Don't free the TargetData, it's owned by the exec engine */
-#else
-   if (gallivm->target) {
-      LLVMDisposeTargetData(gallivm->target);
+   FREE(gallivm->module_name);
+
+   if (!USE_MCJIT) {
+      /* Don't free the TargetData, it's owned by the exec engine */
+   } else {
+      if (gallivm->target) {
+         LLVMDisposeTargetData(gallivm->target);
+      }
    }
-#endif
 
    if (gallivm->builder)
       LLVMDisposeBuilder(gallivm->builder);
@@ -204,6 +206,7 @@ gallivm_free_ir(struct gallivm_state *gallivm)
    gallivm->engine = NULL;
    gallivm->target = NULL;
    gallivm->module = NULL;
+   gallivm->module_name = NULL;
    gallivm->passmgr = NULL;
    gallivm->context = NULL;
    gallivm->builder = NULL;
@@ -254,32 +257,32 @@ init_gallivm_engine(struct gallivm_state *gallivm)
       }
    }
 
-#if !USE_MCJIT
-   gallivm->target = LLVMGetExecutionEngineTargetData(gallivm->engine);
-   if (!gallivm->target)
-      goto fail;
-#else
-   if (0) {
-       /*
-        * Dump the data layout strings.
-        */
+   if (!USE_MCJIT) {
+      gallivm->target = LLVMGetExecutionEngineTargetData(gallivm->engine);
+      if (!gallivm->target)
+         goto fail;
+   } else {
+      if (0) {
+          /*
+           * Dump the data layout strings.
+           */
 
-       LLVMTargetDataRef target = LLVMGetExecutionEngineTargetData(gallivm->engine);
-       char *data_layout;
-       char *engine_data_layout;
+          LLVMTargetDataRef target = LLVMGetExecutionEngineTargetData(gallivm->engine);
+          char *data_layout;
+          char *engine_data_layout;
 
-       data_layout = LLVMCopyStringRepOfTargetData(gallivm->target);
-       engine_data_layout = LLVMCopyStringRepOfTargetData(target);
+          data_layout = LLVMCopyStringRepOfTargetData(gallivm->target);
+          engine_data_layout = LLVMCopyStringRepOfTargetData(target);
 
-       if (1) {
-          debug_printf("module target data = %s\n", data_layout);
-          debug_printf("engine target data = %s\n", engine_data_layout);
-       }
+          if (1) {
+             debug_printf("module target data = %s\n", data_layout);
+             debug_printf("engine target data = %s\n", engine_data_layout);
+          }
 
-       free(data_layout);
-       free(engine_data_layout);
+          free(data_layout);
+          free(engine_data_layout);
+      }
    }
-#endif
 
    return TRUE;
 
@@ -307,6 +310,15 @@ init_gallivm_state(struct gallivm_state *gallivm, const char *name,
    if (!gallivm->context)
       goto fail;
 
+   gallivm->module_name = NULL;
+   if (name) {
+      size_t size = strlen(name) + 1;
+      gallivm->module_name = MALLOC(size);
+      if (gallivm->module_name) {
+         memcpy(gallivm->module_name, name, size);
+      }
+   }
+
    gallivm->module = LLVMModuleCreateWithNameInContext(name,
                                                        gallivm->context);
    if (!gallivm->module)
@@ -324,46 +336,46 @@ init_gallivm_state(struct gallivm_state *gallivm, const char *name,
     * complete when MC-JIT is created. So defer the MC-JIT engine creation for
     * now.
     */
-#if !USE_MCJIT
-   if (!init_gallivm_engine(gallivm)) {
-      goto fail;
-   }
-#else
-   /*
-    * MC-JIT engine compiles the module immediately on creation, so we can't
-    * obtain the target data from it.  Instead we create a target data layout
-    * from a string.
-    *
-    * The produced layout strings are not precisely the same, but should make
-    * no difference for the kind of optimization passes we run.
-    *
-    * For reference this is the layout string on x64:
-    *
-    *   e-p:64:64:64-S128-i1:8:8-i8:8:8-i16:16:16-i32:32:32-i64:64:64-f16:16:16-f32:32:32-f64:64:64-v64:64:64-v128:128:128-a0:0:64-s0:64:64-f80:128:128-f128:128:128-n8:16:32:64
-    *
-    * See also:
-    * - http://llvm.org/docs/LangRef.html#datalayout
-    */
+   if (!USE_MCJIT) {
+      if (!init_gallivm_engine(gallivm)) {
+         goto fail;
+      }
+   } else {
+      /*
+       * MC-JIT engine compiles the module immediately on creation, so we can't
+       * obtain the target data from it.  Instead we create a target data layout
+       * from a string.
+       *
+       * The produced layout strings are not precisely the same, but should make
+       * no difference for the kind of optimization passes we run.
+       *
+       * For reference this is the layout string on x64:
+       *
+       *   e-p:64:64:64-S128-i1:8:8-i8:8:8-i16:16:16-i32:32:32-i64:64:64-f16:16:16-f32:32:32-f64:64:64-v64:64:64-v128:128:128-a0:0:64-s0:64:64-f80:128:128-f128:128:128-n8:16:32:64
+       *
+       * See also:
+       * - http://llvm.org/docs/LangRef.html#datalayout
+       */
 
-   {
-      const unsigned pointer_size = 8 * sizeof(void *);
-      char layout[512];
-      util_snprintf(layout, sizeof layout, "%c-p:%u:%u:%u-i64:64:64-a0:0:%u-s0:%u:%u",
+      {
+         const unsigned pointer_size = 8 * sizeof(void *);
+         char layout[512];
+         util_snprintf(layout, sizeof layout, "%c-p:%u:%u:%u-i64:64:64-a0:0:%u-s0:%u:%u",
 #ifdef PIPE_ARCH_LITTLE_ENDIAN
-                    'e', // little endian
+                       'e', // little endian
 #else
-                    'E', // big endian
+                       'E', // big endian
 #endif
-                    pointer_size, pointer_size, pointer_size, // pointer size, abi alignment, preferred alignment
-                    pointer_size, // aggregate preferred alignment
-                    pointer_size, pointer_size); // stack objects abi alignment, preferred alignment
+                       pointer_size, pointer_size, pointer_size, // pointer size, abi alignment, preferred alignment
+                       pointer_size, // aggregate preferred alignment
+                       pointer_size, pointer_size); // stack objects abi alignment, preferred alignment
 
-      gallivm->target = LLVMCreateTargetData(layout);
-      if (!gallivm->target) {
-         return FALSE;
+         gallivm->target = LLVMCreateTargetData(layout);
+         if (!gallivm->target) {
+            return FALSE;
+         }
       }
    }
-#endif
 
    if (!create_pass_manager(gallivm))
       goto fail;
@@ -383,20 +395,35 @@ lp_build_init(void)
    if (gallivm_initialized)
       return TRUE;
 
+   LLVMLinkInMCJIT();
+#if !defined(USE_MCJIT)
+   USE_MCJIT = debug_get_bool_option("GALLIVM_MCJIT", 0);
+   LLVMLinkInJIT();
+#endif
+
 #ifdef DEBUG
    gallivm_debug = debug_get_option_gallivm_debug();
 #endif
 
    lp_set_target_options();
 
-#if USE_MCJIT
-   LLVMLinkInMCJIT();
-#else
-   LLVMLinkInJIT();
-#endif
-
    util_cpu_detect();
 
+   /* For simulating less capable machines */
+#ifdef DEBUG
+   if (debug_get_bool_option("LP_FORCE_SSE2", FALSE)) {
+      assert(util_cpu_caps.has_sse2);
+      util_cpu_caps.has_sse3 = 0;
+      util_cpu_caps.has_ssse3 = 0;
+      util_cpu_caps.has_sse4_1 = 0;
+      util_cpu_caps.has_sse4_2 = 0;
+      util_cpu_caps.has_avx = 0;
+      util_cpu_caps.has_avx2 = 0;
+      util_cpu_caps.has_f16c = 0;
+      util_cpu_caps.has_fma = 0;
+   }
+#endif
+
    /* AMD Bulldozer AVX's throughput is the same as SSE2; and because using
     * 8-wide vector needs more floating ops than 4-wide (due to padding), it is
     * actually more efficient to use 4-wide vectors on this processor.
@@ -428,6 +455,12 @@ lp_build_init(void)
       util_cpu_caps.has_avx = 0;
       util_cpu_caps.has_avx2 = 0;
       util_cpu_caps.has_f16c = 0;
+      util_cpu_caps.has_fma = 0;
+   }
+   if (HAVE_LLVM < 0x0304 || !USE_MCJIT) {
+      /* AVX2 support has only been tested with LLVM 3.4, and it requires
+       * MCJIT. */
+      util_cpu_caps.has_avx2 = 0;
    }
 
 #ifdef PIPE_ARCH_PPC_64
@@ -454,17 +487,6 @@ lp_build_init(void)
 
    gallivm_initialized = TRUE;
 
-#if 0
-   /* For simulating less capable machines */
-   util_cpu_caps.has_sse3 = 0;
-   util_cpu_caps.has_ssse3 = 0;
-   util_cpu_caps.has_sse4_1 = 0;
-   util_cpu_caps.has_sse4_2 = 0;
-   util_cpu_caps.has_avx = 0;
-   util_cpu_caps.has_avx2 = 0;
-   util_cpu_caps.has_f16c = 0;
-#endif
-
    return TRUE;
 }
 
@@ -572,23 +594,27 @@ gallivm_compile_module(struct gallivm_state *gallivm)
    if (gallivm_debug & GALLIVM_DEBUG_PERF) {
       int64_t time_end = os_time_get();
       int time_msec = (int)(time_end - time_begin) / 1000;
+      assert(gallivm->module_name);
       debug_printf("optimizing module %s took %d msec\n",
-                   lp_get_module_id(gallivm->module), time_msec);
+                   gallivm->module_name, time_msec);
    }
 
    /* Dump byte code to a file */
-   if (0) {
-      LLVMWriteBitcodeToFile(gallivm->module, "llvmpipe.bc");
-      debug_printf("llvmpipe.bc written\n");
-      debug_printf("Invoke as \"llc -o - llvmpipe.bc\"\n");
+   if (gallivm_debug & GALLIVM_DEBUG_DUMP_BC) {
+      char filename[256];
+      assert(gallivm->module_name);
+      util_snprintf(filename, sizeof(filename), "ir_%s.bc", gallivm->module_name);
+      LLVMWriteBitcodeToFile(gallivm->module, filename);
+      debug_printf("%s written\n", filename);
+      debug_printf("Invoke as \"llc -o - %s\"\n", filename);
    }
 
-#if USE_MCJIT
-   assert(!gallivm->engine);
-   if (!init_gallivm_engine(gallivm)) {
-      assert(0);
+   if (USE_MCJIT) {
+      assert(!gallivm->engine);
+      if (!init_gallivm_engine(gallivm)) {
+         assert(0);
+      }
    }
-#endif
    assert(gallivm->engine);
 
    ++gallivm->compiled;
@@ -633,13 +659,24 @@ gallivm_jit_function(struct gallivm_state *gallivm,
 {
    void *code;
    func_pointer jit_func;
+   int64_t time_begin = 0;
 
    assert(gallivm->compiled);
    assert(gallivm->engine);
 
+   if (gallivm_debug & GALLIVM_DEBUG_PERF)
+      time_begin = os_time_get();
+
    code = LLVMGetPointerToGlobal(gallivm->engine, func);
    assert(code);
    jit_func = pointer_to_func(code);
 
+   if (gallivm_debug & GALLIVM_DEBUG_PERF) {
+      int64_t time_end = os_time_get();
+      int time_msec = (int)(time_end - time_begin) / 1000;
+      debug_printf("   jitting func %s took %d msec\n",
+                   LLVMGetValueName(func), time_msec);
+   }
+
    return jit_func;
 }
